@@ -19,6 +19,7 @@
 //
 use std::collections::HashMap;
 use std::fmt::Debug;
+use std::path::Path;
 use std::time::*;
 
 use super::schema::TextSchema;
@@ -284,12 +285,12 @@ impl TextReaderService {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn start(config: &TextConfig) -> NodeResult<Self> {
-        if !config.path.exists() {
-            return Err(node_error!("Invalid path {:?}", config.path));
+    pub fn open(path: &Path) -> NodeResult<Self> {
+        if !path.exists() {
+            return Err(node_error!("Invalid path {:?}", path));
         }
         let field_schema = TextSchema::new();
-        let index = Index::open_in_dir(&config.path)?;
+        let index = Index::open_in_dir(path)?;
 
         let reader = index.reader_builder().reload_policy(ReloadPolicy::OnCommit).try_into()?;
 
