@@ -68,7 +68,7 @@ impl Node {
         let metadata_len = metadata.map(|m| m.as_ref().len()).unwrap_or_default();
         HEADER_LEN + svector_len + skey_len + slabels_len + metadata_len
     }
-    pub fn serialize<S, V, T, M>(key: S, vector: V, labels: T, metadata: Option<M>) -> Vec<u8>
+    pub fn serialize<S, V, T, M>(key: S, vector: V, alignment: usize, labels: T, metadata: Option<M>) -> Vec<u8>
     where
         S: AsRef<[u8]>,
         V: AsRef<[u8]>,
@@ -76,11 +76,18 @@ impl Node {
         M: AsRef<[u8]>,
     {
         let mut buf = vec![];
-        Node::serialize_into(&mut buf, key, vector, labels, metadata).unwrap();
+        Node::serialize_into(&mut buf, key, vector, alignment, labels, metadata).unwrap();
         buf
     }
     // labels must be sorted.
-    pub fn serialize_into<W, S, V, T, M>(mut w: W, key: S, vector: V, trie: T, metadata: Option<M>) -> io::Result<()>
+    pub fn serialize_into<W, S, V, T, M>(
+        mut w: W,
+        key: S,
+        vector: V,
+        alignment: usize,
+        trie: T,
+        metadata: Option<M>,
+    ) -> io::Result<()>
     where
         W: io::Write,
         S: AsRef<[u8]>,
@@ -99,11 +106,17 @@ impl Node {
         let metadata_len = metadata.as_ref().map(|m| m.as_ref().len()).unwrap_or_default();
 
         // Pointer computations
-        let len = HEADER_LEN + svector_len + skey_len + slabels_len + metadata_len;
         let vector_start = HEADER_LEN + metadata_len;
-        let key_start = vector_start + svector_len;
+        let vector_pad = if vector_start % alignment > 0 {
+            alignment - (vector_start % alignment)
+        } else {
+            0
+        };
+        let key_start = vector_start + svector_len + vector_pad;
         let labels_start = key_start + skey_len;
 
+        let len = HEADER_LEN + vector_pad + svector_len + skey_len + slabels_len + metadata_len;
+
         // Write pointers
         w.write_all(&len.to_le_bytes())?;
         w.write_all(&vector_start.to_le_bytes())?;
@@ -112,7 +125,9 @@ impl Node {
         // Metadata segment
         metadata.map_or(Ok(()), |m| w.write_all(m.as_ref()))?;
         // Values
-        w.write_all(&svector.len().to_le_bytes())?;
+        w.write_all(&(svector.len() as u32).to_le_bytes())?;
+        w.write_all(&(vector_pad as u32).to_le_bytes())?;
+        w.write_all(&[0].repeat(vector_pad))?;
         w.write_all(svector)?;
         w.write_all(&skey.len().to_le_bytes())?;
         w.write_all(skey)?;
@@ -144,8 +159,9 @@ impl Node {
     // x must be serialized using Node, may have trailing bytes.
     pub fn vector(x: &[u8]) -> &[u8] {
         let xvec_ptr = usize_from_slice_le(&x[VECTOR_START.0..VECTOR_START.1]);
-        let xvec_len = usize_from_slice_le(&x[xvec_ptr..(xvec_ptr + USIZE_LEN)]);
-        let xvec_start = xvec_ptr + USIZE_LEN;
+        let xvec_len = u32_from_slice_le(&x[xvec_ptr..(xvec_ptr + U32_LEN)]) as usize;
+        let xvec_pad = u32_from_slice_le(&x[(xvec_ptr + U32_LEN)..(xvec_ptr + 2 * U32_LEN)]);
+        let xvec_start = xvec_ptr + 2 * U32_LEN + xvec_pad as usize;
         &x[xvec_start..(xvec_start + xvec_len)]
     }
     // x must be serialized using Node, may have trailing bytes.
@@ -167,7 +183,7 @@ impl Interpreter for Node {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::data_types::{trie_ram, vector};
+    use crate::{data_types::trie_ram, vector_types::dense_f32_unaligned};
     lazy_static::lazy_static! {
         static ref NO_LABELS_TRIE: Vec<u8> = trie::serialize(trie_ram::create_trie(&NO_LABELS));
         static ref LABELS_TRIE: Vec<u8> = trie::serialize(trie_ram::create_trie(&LABELS));
@@ -179,9 +195,9 @@ mod tests {
     #[test]
     fn create_test() {
         let key = b"NODE1";
-        let vector = vector::encode_vector(&[12.; 1000]);
+        let vector = dense_f32_unaligned::encode_vector(&[12.; 1000]);
         let mut buf = Vec::new();
-        Node::serialize_into(&mut buf, key, &vector, NO_LABELS_TRIE.clone(), NO_METADATA).unwrap();
+        Node::serialize_into(&mut buf, key, &vector, 1, NO_LABELS_TRIE.clone(), NO_METADATA).unwrap();
         let len = usize_from_slice_le(&buf[LEN.0..LEN.1]);
         let vector_start = usize_from_slice_le(&buf[VECTOR_START.0..VECTOR_START.1]);
         let key_start = usize_from_slice_le(&buf[KEY_START.0..KEY_START.1]);
@@ -201,9 +217,9 @@ mod tests {
 
         let key = b"NODE2";
         let metadata = b"THIS ARE THE METADATA CONTENTS";
-        let vector = vector::encode_vector(&[13.; 1000]);
+        let vector = dense_f32_unaligned::encode_vector(&[13.; 1000]);
         let mut buf = Vec::new();
-        Node::serialize_into(&mut buf, key, &vector, LABELS_TRIE.clone(), Some(metadata)).unwrap();
+        Node::serialize_into(&mut buf, key, &vector, 1, LABELS_TRIE.clone(), Some(metadata)).unwrap();
         let len = usize_from_slice_le(&buf[LEN.0..LEN.1]);
         let vector_start = usize_from_slice_le(&buf[VECTOR_START.0..VECTOR_START.1]);
         let key_start = usize_from_slice_le(&buf[KEY_START.0..KEY_START.1]);
@@ -228,14 +244,14 @@ mod tests {
         let mut buf = Vec::new();
         let key1 = b"NODE1";
         let metadata1 = b"The node 1 has metadata";
-        let vector1 = vector::encode_vector(&[12.; 1000]);
+        let vector1 = dense_f32_unaligned::encode_vector(&[12.; 1000]);
         let node1 = buf.len();
-        Node::serialize_into(&mut buf, key1, &vector1, NO_LABELS_TRIE.clone(), Some(&metadata1)).unwrap();
+        Node::serialize_into(&mut buf, key1, &vector1, 1, NO_LABELS_TRIE.clone(), Some(&metadata1)).unwrap();
         let key2 = b"NODE2";
         let metadata2 = b"Tuns out node 2 also has metadata";
-        let vector2 = vector::encode_vector(&[15.; 1000]);
+        let vector2 = dense_f32_unaligned::encode_vector(&[15.; 1000]);
         let node2 = buf.len();
-        Node::serialize_into(&mut buf, key2, &vector2, NO_LABELS_TRIE.clone(), Some(&metadata2)).unwrap();
+        Node::serialize_into(&mut buf, key2, &vector2, 1, NO_LABELS_TRIE.clone(), Some(&metadata2)).unwrap();
         assert_eq!(Node::key(&buf[node1..]), key1);
         assert_eq!(Node::key(&buf[node2..]), key2);
         assert_eq!(Node::vector(&buf[node1..]), vector1);
