@@ -18,60 +18,105 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 
-use crate::data_point::{Address, DataRetriever};
+use std::collections::HashSet;
 
-#[derive(Debug, Clone, serde::Serialize, serde::Deserialize, PartialEq)]
-pub enum AtomKind {
-    KeyPrefix,
-    Label,
-}
+use crate::data_point::{Address, DataRetriever};
 
 /// Is a singleton clause.
 #[derive(Debug, Clone, serde::Serialize, serde::Deserialize, PartialEq)]
-pub struct AtomClause {
-    kind: AtomKind,
-    value: String,
+pub enum AtomClause {
+    KeyPrefix(String),
+    Label(String),
+    KeyPrefixSet((HashSet<String>, HashSet<String>)),
 }
 impl AtomClause {
-    pub fn new(value: String, kind: AtomKind) -> AtomClause {
-        AtomClause { kind, value }
-    }
     pub fn label(value: String) -> AtomClause {
-        AtomClause::new(value, AtomKind::Label)
+        Self::Label(value)
     }
     pub fn key_prefix(value: String) -> AtomClause {
-        AtomClause::new(value, AtomKind::KeyPrefix)
+        Self::KeyPrefix(value)
+    }
+    pub fn key_set(resource_set: HashSet<String>, field_set: HashSet<String>) -> AtomClause {
+        Self::KeyPrefixSet((resource_set, field_set))
     }
     fn run<D: DataRetriever>(&self, x: Address, retriever: &D) -> bool {
-        match self.kind {
-            AtomKind::KeyPrefix => retriever.get_key(x).starts_with(self.value.as_bytes()),
-            AtomKind::Label => retriever.has_label(x, self.value.as_bytes()),
+        match self {
+            Self::KeyPrefix(value) => retriever.get_key(x).starts_with(value.as_bytes()),
+
+            Self::Label(value) => retriever.has_label(x, value.as_bytes()),
+            Self::KeyPrefixSet((resource_set, field_set)) => {
+                let key = retriever.get_key(x);
+                let mut key_parts = key.split(|b| *b == b'/');
+
+                // Matches resource_id
+                let resource_id = std::str::from_utf8(key_parts.next().unwrap()).unwrap();
+                let matches_resource = resource_set.contains(resource_id);
+                if matches_resource {
+                    return true;
+                }
+
+                // Matches field_id (key up to the third slash)
+                let mut slash_count = 0;
+                let mut end_pos = 0;
+                for char in key.iter() {
+                    if *char == b'/' {
+                        slash_count += 1;
+                        if slash_count == 3 {
+                            break;
+                        }
+                    }
+                    end_pos += 1;
+                }
+                // slash_count = 2 if we reach the end of string with 2 middle slashes
+                if slash_count < 2 {
+                    return false;
+                }
+
+                field_set.contains(std::str::from_utf8(&key[0..end_pos]).unwrap())
+            }
         }
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Deserialize, serde::Serialize)]
+pub enum BooleanOperator {
+    Not,
+    Or,
+    And,
+}
+
 /// Is a clause formed by the conjuction of several LabelClauses. Additionally this
 /// clause has a threshold that specifies the minimum number of AtomClauses that have to
 /// succeed in order for the overall conjuction to be satisfied.
-#[derive(Debug, Clone, Default, serde::Serialize, serde::Deserialize, PartialEq)]
+#[derive(Debug, Clone, serde::Serialize, serde::Deserialize, PartialEq)]
 pub struct CompoundClause {
-    labels: Vec<AtomClause>,
+    operator: BooleanOperator,
+    operands: Vec<Clause>,
 }
 impl CompoundClause {
     pub fn len(&self) -> usize {
-        self.labels.len()
+        self.operands.len()
     }
     pub fn is_empty(&self) -> bool {
-        self.labels.is_empty()
+        self.operands.is_empty()
     }
-    pub fn new(labels: Vec<AtomClause>) -> CompoundClause {
-        CompoundClause { labels }
+    pub fn new(operator: BooleanOperator, operands: Vec<Clause>) -> CompoundClause {
+        CompoundClause {
+            operator,
+            operands,
+        }
     }
     fn run<D: DataRetriever>(&self, x: Address, retriever: &D) -> bool {
         if self.is_empty() {
             return true;
         }
-        self.labels.iter().any(|label| label.run(x, retriever))
+
+        let mut subquery_iterator = self.operands.iter();
+        match self.operator {
+            BooleanOperator::And => subquery_iterator.all(|subquery| subquery.run(x, retriever)),
+            BooleanOperator::Or => subquery_iterator.any(|subquery| subquery.run(x, retriever)),
+            BooleanOperator::Not => !subquery_iterator.all(|subquery| subquery.run(x, retriever)),
+        }
     }
 }
 
@@ -103,23 +148,7 @@ impl From<CompoundClause> for Clause {
     }
 }
 
-#[derive(Default)]
-pub struct AtomCollector {
-    pub labels: Vec<String>,
-    pub key_prefixes: Vec<String>,
-}
-impl AtomCollector {
-    fn add(&mut self, atom: AtomClause) {
-        match atom.kind {
-            AtomKind::KeyPrefix => self.key_prefixes.push(atom.value),
-            AtomKind::Label => self.labels.push(atom.value),
-        }
-    }
-}
-
-/// Formulas are boolean expressions in conjuctive normal form, but for labels.
-/// The clauses in a formula are connected by intersections, and they are formed
-/// by strings. Once applied to a given address, the formula becomes a boolean
+/// Once applied to a given address, the formula becomes a boolean
 /// expression that evaluates to whether the address is valid or not.
 #[derive(Debug, Clone, Default, serde::Serialize, serde::Deserialize, PartialEq)]
 pub struct Formula {
@@ -130,27 +159,14 @@ impl Formula {
         Formula::default()
     }
     pub fn extend<C>(&mut self, clause: C)
-    where Clause: From<C> {
+    where
+        Clause: From<C>,
+    {
         self.clauses.push(clause.into())
     }
     pub fn run<D: DataRetriever>(&self, x: Address, retriever: &D) -> bool {
         self.clauses.iter().all(|q| q.run(x, retriever))
     }
-    /// Returns the atoms that form a formula
-    pub fn get_atoms(&self) -> AtomCollector {
-        let mut collector = AtomCollector::default();
-        for clause in self.clauses.iter() {
-            match clause {
-                Clause::Compound(q) => {
-                    for label in q.labels.iter() {
-                        collector.add(label.clone());
-                    }
-                }
-                Clause::Atom(q) => collector.add(q.clone()),
-            }
-        }
-        collector
-    }
 }
 
 #[cfg(test)]
@@ -182,6 +198,9 @@ mod tests {
         fn min_score(&self) -> f32 {
             -1.0
         }
+        fn will_need(&self, _x: Address) {
+            // noop
+        }
     }
     #[test]
     fn test_query() {
@@ -204,22 +223,74 @@ mod tests {
         formula.extend(AtomClause::label(L2.to_string()));
         assert!(!formula.run(ADDRESS, &retriever));
 
-        let mut formula = Formula::new();
-        let inner = vec![
-            AtomClause::label(L1.to_string()),
-            AtomClause::label(L2.to_string()),
+        #[rustfmt::skip] let inner = vec![
+            Clause::Atom(AtomClause::label(L1.to_string())), 
+            Clause::Atom(AtomClause::label(L2.to_string()))
         ];
-        formula.extend(CompoundClause::new(inner));
+        let mut formula = Formula::new();
+        formula.extend(CompoundClause::new(BooleanOperator::Or, inner));
         assert!(formula.run(ADDRESS, &retriever));
 
         let mut formula = Formula::new();
-        let inner = vec![AtomClause::key_prefix("/This/is".to_string())];
-        formula.extend(CompoundClause::new(inner));
+        let inner = vec![Clause::Atom(AtomClause::key_prefix("/This/is".to_string()))];
+        formula.extend(CompoundClause::new(BooleanOperator::Or, inner));
         assert!(formula.run(ADDRESS, &retriever));
         let mut formula = Formula::new();
 
-        let inner = vec![AtomClause::key_prefix("/This/is/not".to_string())];
-        formula.extend(CompoundClause::new(inner));
+        let inner = vec![Clause::Atom(AtomClause::key_prefix("/This/is/not".to_string()))];
+        formula.extend(CompoundClause::new(BooleanOperator::Or, inner));
+        assert!(!formula.run(ADDRESS, &retriever));
+    }
+
+    #[test]
+    fn test_key_filters() {
+        let resource_id = String::from("015163a0629e4f368aa9d54978d2a9ff");
+        const FIELD_ID: &str = "015163a0629e4f368aa9d54978d2a9ff/f/file1";
+        let field_id = String::from(FIELD_ID);
+
+        let fake_resource_id = String::from("badcafe0badcafe0badcafe0badcafe0");
+        let fake_field_id = format!("{resource_id}/f/not_a_field");
+
+        const ADDRESS: Address = Address::dummy();
+        let retriever = DummyRetriever {
+            key: FIELD_ID.as_bytes(),
+            labels: HashSet::new(),
+        };
+
+        // Find by resource_id
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_prefix(resource_id.clone()));
+        assert!(formula.run(ADDRESS, &retriever));
+
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_prefix(fake_resource_id.clone()));
+        assert!(!formula.run(ADDRESS, &retriever));
+
+        // Find by field_id
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_prefix(field_id.clone()));
+        assert!(formula.run(ADDRESS, &retriever));
+
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_prefix(fake_field_id.clone()));
+        assert!(!formula.run(ADDRESS, &retriever));
+
+        // Find by set of resource_ids
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_set([resource_id.clone()].into_iter().collect(), HashSet::new()));
+        assert!(formula.run(ADDRESS, &retriever));
+
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_set([fake_resource_id.clone()].into_iter().collect(), HashSet::new()));
+        assert!(!formula.run(ADDRESS, &retriever));
+
+        // Find by set of field_ids
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_set(HashSet::new(), [field_id.clone()].into_iter().collect()));
+        assert!(formula.run(ADDRESS, &retriever));
+
+        let mut formula = Formula::new();
+        formula.extend(AtomClause::key_set(HashSet::new(), [fake_field_id.clone()].into_iter().collect()));
         assert!(!formula.run(ADDRESS, &retriever));
     }
 }
